/*
 *  linux/include/asm-arm/arch-pxa/mainstone.h
 *
 *  Orignal Author:	Nicolas Pitre
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#define MST_FPGA_PHYS	PXA_CS2_PHYS	
#define MST_FPGA_VIRT	(0xf0000000)	/* phys 0x08000000 */
#define MST_ETH_PHYS	(0x0c000000)	 
#define MST_ETH_BASE	(0xf1000000)	/* phys 0x0c000000 */

#define MST_P2V(x)		((x) - MST_FPGA_PHYS + MST_FPGA_VIRT)
#define MST_V2P(x)		((x) - MST_FPGA_VIRT + MST_FPGA_PHYS)

#ifndef __ASSEMBLY__
#  define __MST_REG(x)		(*((volatile unsigned long *)MST_P2V(x)))
#else
#  define __MST_REG(x)		MST_P2V(x)
#endif

/* board level registers in the CPLD: (offsets from CPLD_BASE) */

#define WHOAMI			0	// card ID's (see programmers manual)
#define HEX_LED			0x10	// R/W access to 8 7 segment displays
#define DISC_BLNK_LED		0x40 	// R/W [15-8] enables for hex leds, [7-0] discrete LEDs
#define CONF_SWITCHES		0x50	// RO [1] flash wrt prot, [0] 0= boot from rom, 1= flash
#define USER_SWITCHES		0x60	// RO [15-8] dip switches, [7-0] 2 hex encoding switches
#define MISC_WR			0x80	// R/W various system controls -see manual
#define MISC_RD			0x90	// RO various system status bits -see manual
//#define MST_IRQ_MASK_EN		0xC0    // R/W 0= mask, 1= enable of TS, codec, ethernet, USB, SA1111, and card det. irq's
//#define MST_IRQ_SET_CLR		0xD0	// R/W 1= set, 0 = clear IRQ's from TS, codec, etc...
//#define MST_GP			0x100	// R/W [15-0] 16 bits of general purpose I/o for hacking


/* FPGA register physical addresses */
//#define _MST_WHOAMI		(MST_FPGA_PHYS + 0x000)
#define _MST_LEDDAT1		(MST_FPGA_PHYS + 0x010)
#define _MST_LEDDAT2		(MST_FPGA_PHYS + 0x014)
#define _MST_LEDCTRL		(MST_FPGA_PHYS + 0x040)
#define _MST_GPSWR		(MST_FPGA_PHYS + 0x060)
#define _MST_MSCWR1		(MST_FPGA_PHYS + 0x080)
#define _MST_MSCWR2		(MST_FPGA_PHYS + 0x084)
#define _MST_MSCWR3		(MST_FPGA_PHYS + 0x088)
#define _MST_MSCRD		(MST_FPGA_PHYS + 0x090)
#define _MST_IRQMSKENA		(MST_FPGA_PHYS + 0x0C0)
#define _MST_IRQSETCLR		(MST_FPGA_PHYS + 0x0D0)
//#define _MST_GP			(MST_FPGA_PHYS + 0x100)

/* FPGA register virtual addresses */
//#define MST_WHOAMI		__MST_REG(_MST_WHOAMI)
#define MST_LEDDAT1		__MST_REG(_MST_LEDDAT1)		
#define MST_LEDDAT2		__MST_REG(_MST_LEDDAT2)		
#define MST_LEDCTRL		__MST_REG(_MST_LEDCTRL)	
#define MST_GPSWR		__MST_REG(_MST_GPSWR)
#define MST_MSCWR1		__MST_REG(_MST_MSCWR1)	
#define MST_MSCWR2		__MST_REG(_MST_MSCWR2)	
#define MST_MSCWR3		__MST_REG(_MST_MSCWR3)
#define MST_MSCRD		__MST_REG(_MST_MSCRD)		
#define MST_IRQMSKENA		__MST_REG(_MST_IRQMSKENA)
#define MST_IRQSETCLR		__MST_REG(_MST_IRQSETCLR)		
//#define MST_GP			__MST_REG(_MST_GP)	

/* Register bits */

#define MSCWR1_CAMERA_ON	(0x1 << 15)	/* Camera Interface Power Control */
#define MSCWR1_CAMERA_SEL	(0x1 << 14)	/* Camera Interface Mux control */
#define MSCWR1_LCD_CTL		(0x1 << 13)	/* General Purpose LCD control signal */
#define MSCWR1_MS_ON		(0x1 << 12)	/* Memory Stick power control */
#define MSCWR1_MMC_ON		(0x1 << 11)	/* MultiMediaCard Power control */
#define MSCWR1_MS_SEL		(0x1 << 10)	/* SD/Memory Stick multiplexer control */
#define MSCWR1_BB_SEL		(0x1 << 9)	/* PCMCIA/BaseBand mulitplexer control */
#define MSCWR1_nBT_OFF		(0x1 << 8)	/* Bluetooth UART transceiver contrl */
#define MSCWR1_BTDTR		(0x1 << 7)	/* Bluetooth UART Data Terminal Ready(DTR) */
#define MSCWR1_IrDA_FIR		(0x1 << 4)	/* IrDA transceiver SIR/FIR control */
#define MSCWR1_GREENLED		(0x1 << 3)	/* LCD D1 control */
#define MSCWR1_PDC_CTL		(0x1 << 2)	/* reserved */
#define MSCWR1_MTR_ON		(0x1 << 1)	/* Silent alert motor control */
#define MSCWR1_SYSRESET		(0x1 << 0)	/* System reset */

#define MSCWR2_nLEGACY_SEL	(0x1 << 8)	/* Legacy Mode control */
#define MSCWR2_GRAPHICS_SEL	(0x1 << 7)	/* Marathon Graphics Accelerator control */
#define MSCWR2_USB_OTG_RST	(0x1 << 6)	/* USB On The Go External Transceiver Reset */
#define MSCWR2_USB_OTG_SEL	(0x1 << 5)	/* USB On The GO Control */
#define MSCWR2_nUSBC_SC		(0x1 << 4)	/* USB client soft connect control */
#define MSCWR2_LINE2		(0x1 << 3)	/* Line Out 2 amplifer control */
#define MSCWR2_LINE1		(0x1 << 2)	/* Line Out 1 amplifer control */
#define MSCWR2_RADIO_PWR	(0x1 << 1)	/* Radio module power control */
#define MSCWR2_RADIO_WAKE	(0x1 << 0)	/* Radio module wake-up signal */

#define MSCWR3_GPIO_RESET_EN	(0x1 << 2)	/* Enable GPIO Resets */
#define MSCWR3_GPIO_RESET 	(0x1 << 1)	/* Initiate GPIO Reset */
#define MSCWR3_COMMS_SW_RESET 	(0x1 << 0)	/* Communications Processor Reset control */

#define MSCRD1_nFPG_PRES	(0x1 << 11)	/* Processor Card Present */
#define MSCRD1_POLL_FLAG	(0x1 << 10)	/* Marathon Poll Flag signal */
#define MSCRD1_nPENIRQ		(0x1 << 9)	/* ADI7873 touch screen digitizer nPENIRQ singal */
#define MSCRD1_nMEMSTK_CD	(0x1 << 8)	/* Memory Stick detection signal */
#define MSCRD1_nMMC_CD		(0x1 << 7)	/* SD/MMC card detection singal */
#define MSCRD1_uUSIM_CD		(0x1 << 6)	/* USIM card detection signal */
#define MSCRD1_USB_CBL		(0x1 << 5)	/* USB client cable status */
#define MSCRD1_TS_BUSY		(0x1 << 4)	/* ADI7873 touch-screen digitizer busy */
#define MSCRD1_BTDSR		(0x1 << 3)	/* Bluetooth UART Data Set Ready */
#define MSCRD1_BTRI		(0x1 << 2)	/* Bluetooth UART Ring Indictor */
#define MSCRD1_BTDCD		(0x1 << 1)	/* Bluetooth UART Data Carrier Detect */
#define MSCRD1_nMMC_WP		(0x1 << 0)	/* SD/MMC write-protect status */

/* GPIOs */

#define GPIO_MAINSTONE_IRQ	0
#define IRQ_GPIO_MAINSTONE_IRQ	IRQ_GPIO0


/*
 * LED macros
 */

#define LEDS_BASE MST_LEDCTRL

// 8 discrete leds available for general use:

#define D28	0x1
#define D27	0x2
#define D26	0x4
#define D25	0x8
#define D24	0x10
#define D23	0x20
#define D22	0x40
#define D21	0x80

/* Note: bits [15-8] are used to enable/blank the 8 7 segment hex displays so
*  be sure to not monkey with them here.
*/

#define HEARTBEAT_LED	D28
#define SYS_BUSY_LED    D27
#define HEXLEDS_BASE MST_HEXLED

#define HEARTBEAT_LED_ON  (LEDS_BASE &= ~HEARTBEAT_LED)
#define HEARTBEAT_LED_OFF (LEDS_BASE |= HEARTBEAT_LED)
#define SYS_BUSY_LED_OFF  (LEDS_BASE |= SYS_BUSY_LED)
#define SYS_BUSY_LED_ON   (LEDS_BASE &= ~SYS_BUSY_LED)

// use x = D26-D21 for these, please...
#define DISCRETE_LED_ON(x) (LEDS_BASE &= ~(x))
#define DISCRETE_LED_OFF(x) (LEDS_BASE |= (x))

#ifndef __ASSEMBLY__

//extern int hexled_val = 0;

#endif

#define BUMP_COUNTER (HEXLEDS_BASE = hexled_val++)
#define DEC_COUNTER (HEXLEDS_BASE = hexled_val--)
